-- lua/gemini/api.lua

local M = {}

-- Store conversation history
local conversation_history = {}

-- Helper function to get API key
local function get_api_key()
	return vim.g.gemini_api_key or os.getenv("GEMINI_API_KEY")
end

-- Async HTTP request function
local function async_request(url, payload, callback)
	local response = ""
	local error_msg = ""

	-- Create pipes for stdout and stderr
	local stdout = vim.loop.new_pipe()
	local stderr = vim.loop.new_pipe()
	
	-- Spawn curl process with handle in broader scope
	local handle
	local function cleanup()
		if stdout then stdout:close() end
		if stderr then stderr:close() end
		if handle then handle:close() end
	end

	handle = vim.loop.spawn('curl', {
		args = {
			'-s',
			'-X', 'POST',
			'-H', 'Content-Type: application/json',
			'-d', payload,
			url
		},
		stdio = {nil, stdout, stderr}
	}, function(exit_code, signal)  -- This is the exit callback
		-- Clean up handles
		cleanup()

		if exit_code ~= 0 then
			vim.schedule(function()
				callback(nil, "Curl failed with code " .. exit_code .. ": " .. error_msg)
			end)
			return
		end

		-- Try to parse the response
		local success, decoded = pcall(vim.json.decode, response)
		vim.schedule(function()
			if success then
				callback(decoded)
			else
				callback(nil, "JSON decode error: " .. response)
			end
		end)
	end)

	if not handle then
		cleanup()
		vim.schedule(function()
			callback(nil, "Failed to start curl")
		end)
		return
	end

	-- Handle stdout data
	stdout:read_start(function(err, chunk)
		if err then
			cleanup()
			vim.schedule(function()
				callback(nil, "Read error: " .. err)
			end)
			return
		end
		if chunk then
			response = response .. chunk
		end
	end)

	-- Handle stderr data
	stderr:read_start(function(err, chunk)
		if err then
			cleanup()
			vim.schedule(function()
				callback(nil, "Error reading stderr: " .. err)
			end)
			return
		end
		if chunk then
			error_msg = error_msg .. chunk
		end
	end)
end

function M.get_response(prompt, context, callback)
	local api_key = get_api_key()
	
	if not api_key then
		vim.schedule(function()
			callback(nil, "API key not set")
		end)
		return
	end

	local model = "gemini-2.0-flash"
	local contents = {}
	
	-- Add conversation history
	for _, message in ipairs(conversation_history) do
		table.insert(contents, {
			role = message.role,
			parts = {{
				text = message.content
			}}
		})
	end

	-- Add current prompt
	if context then
		table.insert(contents, {
			role = "user",
			parts = {{
				text = "Context:\n" .. context .. "\n\nQuery: " .. prompt
			}}
		})
	else
		table.insert(contents, {
			role = "user",
			parts = {{
				text = prompt
			}}
		})
	end

	-- Store prompt in history
	table.insert(conversation_history, {
		role = "user",
		content = prompt
	})

	local payload = vim.json.encode({
		contents = contents,
	})

	local url = string.format(
		"https://generativelanguage.googleapis.com/v1/models/%s:generateContent?key=%s",
		model,
		api_key
	)

	async_request(url, payload, function(result, error)
		if error then
			callback(nil, error)
			return
		end

		if result.error then
			callback(nil, "API Error: " .. vim.inspect(result.error))
			return
		end

		if result.candidates and
		   result.candidates[1] and
		   result.candidates[1].content and
		   result.candidates[1].content.parts and
		   result.candidates[1].content.parts[1] and
		   result.candidates[1].content.parts[1].text then
			
			local response_text = result.candidates[1].content.parts[1].text
			
			-- Store response in history
			table.insert(conversation_history, {
				role = "model",
				content = response_text
			})
			
			callback(response_text)
		else
			callback(nil, "Unexpected response structure")
		end
	end)
end

-- Add function to clear conversation history
function M.clear_conversation()
	conversation_history = {}
end

return M
